/*
  Copyright 2008 CNRS
  Author: Florent Lamiraux
*/

#ifndef HPPWALKDISTANCE_H
#define HPPWALKDISTANCE_H

#include "KineoWorks2/kwsDistance.h"
KIT_PREDEF_CLASS(CflicDistance);
KIT_PREDEF_CLASS(ChppWalkDistance);

/**
   \brief Distance function relative to ChppWalkSteeringMethod
*/

class ChppWalkDistance : public CkwsDistance
{
public:
  
  virtual ~ChppWalkDistance();

  /**
     \brief Return a shared pointer to a distance object
  */
  static ChppWalkDistanceShPtr create(double inDistanceThreshold);

  /**
     \brief Return a shared pointer to a distance object
  */
  static ChppWalkDistanceShPtr createCopy(const ChppWalkDistanceConstShPtr& inDistance);

  /**
     \brief Compute an approximation of the distance between two configurations
     \param inConfig1 first configuration.
     \param inConfig2 second configuration.

     \return Length of direct path created by steering method ChppWalkSteeringMethod.
  */
  double distance(const CkwsConfig &inConfig1, const CkwsConfig &inConfig2) const;

  /**
     \brief Cost of direct path
     \param inDirectPath direct path to evaluate

     This method is used to find the best path in a roadmap or to optimize a path.
  */
  virtual double cost (const CkwsDirectPathConstShPtr &inDirectPath) const;

protected:
  /**
     \brief initialization: store weak pointer to itself
  */
  ktStatus init(const ChppWalkDistanceWkPtr &inWeakPtr);

  /**
     \brief Protected constructor
  */
  ChppWalkDistance(double inDistanceThreshold);

  /**
     \brief Protected copy constructor
  */
  ChppWalkDistance(const ChppWalkDistance& inDistance);

private:

  /**
     \brief Store weak pointer to object
  */
  ChppWalkDistanceWkPtr attWeakPtr;

  /**
     \brief flic distance function
  */
  CflicDistanceShPtr attFlicDistance;

  /**
     \brief Linear distance function
  */
  CkwsDistanceShPtr attLinearDistance;

  /**
     \brief Distance above which linear direct path is not allowed
  */
  double attDistanceThreshold;

};


#endif
