#ifndef HPPWALKPLANNERDELEGATE_H
#define HPPWALKPLANNERDELEGATE_H

#include "KineoWorks2/kwsRdmBuilderDelegate.h"

KIT_PREDEF_CLASS(CkwsSteeringMethod);

/**
   \brief This class implements a roadmap builder delegate for walk planning

   Each time node is inserted, 
   \li the delegates attempts at inserting the same node with opposite orientation (half-turn). 
   \li in case of success a map is updated in order to keep track of current node counterpart.
   \li the delegate attempts at inserting an edge between the node and its counterpart.

*/

class ChppWalkPlannerDelegate : public CkwsRdmBuilderDelegate
{
public:
  /**
     \brief Constructor
  */
  ChppWalkPlannerDelegate(CkwsRoadmapBuilderWkPtr inRdmBuilder) : 
    CkwsRdmBuilderDelegate(),
    attRdmBuilder(inRdmBuilder),
    attSteeringMethod(),
    nodeAddedByThisDelegate(false),
    edgeAddedByThisDelegate(false) {};

  /**
     \brief Set steering method
  */
  void steeringMethod(CkwsSteeringMethodShPtr inSteeringMethod) {
    attSteeringMethod = inSteeringMethod;
  };

  /**
     \brief Method called when a node is added to the roadmap.
  */
  void builderDidAddNode (const CkwsRoadmapBuilderConstShPtr &inBuilder, const CkwsNodeConstShPtr &inNode);

  /**
     \brief Method called when an edge is added
  */
  void builderDidAddEdge (const CkwsRoadmapBuilderConstShPtr &inBuilder, const CkwsEdgeConstShPtr &inEdge);

private:

  /**
     \brief Weak pointer to the roadmap builder owning the delegate
  */
  CkwsRoadmapBuilderWkPtr attRdmBuilder;

  /**
     \brief Steering method used to add edges

     Default is steering method of the device.
  */
  CkwsSteeringMethodShPtr attSteeringMethod;

  /**
     \brief Flag specifying that the delegate added a node.
     
     If this flag is true, method builderDidAddNode does not treat the event.
  */
  bool nodeAddedByThisDelegate;

  /**
     \brief Flag specifying that the delegate added an edge.
     
     If this flag is true, method builderDidAddEdge does not treat the event.
  */
  bool edgeAddedByThisDelegate;
};

#endif
