/*
  Copyright 2008 CNRS
  Author: Florent Lamiraux
*/

#include "kwsIO/kwsioConfig.h"
#include "KineoWorks2/kwsDistance.h"
#include "hppWalkFootPlanner/hppWalkSteeringMethod.h"
#include "kwsPlus/directPath/flicDirectPath.h"
#include "kwsPlus/directPath/kwsPlusDPLinear.h"
#include "kwsPlus/directPath/kwsPlusSMLinear.h"

// Select level of verbosity at configuration.
#if DEBUG==2
#define ODEBUG2(x) std::cout << "ChppWalkSteeringMethod:" << x << std::endl
#define ODEBUG1(x) std::cerr << "ChppWalkSteeringMethod:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "ChppWalkSteeringMethod:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

ChppWalkSteeringMethod::ChppWalkSteeringMethod(double inDistanceThreshold) :
  CkwsSteeringMethod(), attDistanceThreshold(inDistanceThreshold)
{
}

ChppWalkSteeringMethod::~ChppWalkSteeringMethod()
{
}

ChppWalkSteeringMethodShPtr ChppWalkSteeringMethod::create(double inDistanceThreshold)
{
  ChppWalkSteeringMethod* smPtr = new ChppWalkSteeringMethod(inDistanceThreshold);
  ChppWalkSteeringMethodShPtr smShPtr(smPtr);
  ChppWalkSteeringMethodWkPtr smWkPtr(smShPtr);

  if (smPtr->init(smWkPtr) != KD_OK) {
    smShPtr.reset();
  }
  return smShPtr;
}



CkwsDirectPathShPtr ChppWalkSteeringMethod::makeDirectPath(const CkwsConfig& inStartCfg, 
							   const CkwsConfig& inEndCfg)
{
  CkwsDirectPathShPtr path = CflicDirectPath::create(inStartCfg, inEndCfg, 
						     attWeakPtr.lock(), true);

  if (path) {
    return path;
  }

  /*
    If flic failed, and configurations are close to each other,
    try linear direct path
  */
  if (attDistance->distance(inStartCfg, inEndCfg) <= attDistanceThreshold) {
    path = CkwsPlusDPLinear::create(inStartCfg, inEndCfg, std::vector<double>(0), 
				    attWeakPtr.lock());
  }
  return path;
}


ktStatus ChppWalkSteeringMethod::init(const CkwsSteeringMethodWkPtr& inSteeringMethodWkPtr)
{
  ktStatus  success = CkwsSteeringMethod::init(inSteeringMethodWkPtr);

  if(KD_OK == success) {
    attWeakPtr = inSteeringMethodWkPtr;
  }
  attDistance = CkwsDistance::create();

  if (!attDistance) {
    success = KD_ERROR;
  }
  if (success == KD_ERROR) {
    ODEBUG1(":init: failed.");
  }
  return success;
}

